<form id="user-form">
	<div class="inside-wrapper">
		<table class="normalrightcolumn">
			<tr>
				<th><label for="name">Name</label></th>
				<td><input type="text" id="name" name="name" value="<%-values.name ? values.name : ''%>" /></td>
			</tr>
			<tr class="even">
				<th><label for="password">Password</label></th>
				<td><input type="text" id="password" name="password" value="<%-values.password ? values.password : ''%>" /><%-action != 'edit' ? '' : 'Leave empty, if you want to keep current password.'%></td>
			</tr>
			<tr>
				<th><label for="permissions">Permissions</label></th>
				<td>
					<p><a href="#" id="add-permissions">Add more permissions</a></p>
					<table id="permissions">
						<tr>
							<td>Group</td>
							<td>Remove</td>
						</tr>
					</table>
				</td>
			</tr>
			<tr class="even">
				<th>Allowed?</th>
				<td><% if (values.allowed || values.allowed === undefined) {%>
					<label for="allowed">YES</label><input type="checkbox" value="true" name="allowed" id="allowed" checked="checked" />
					<% } else { %>
					<label for="allowed">NO</label><input type="checkbox" value="true" name="allowed" id="allowed" />
					<% } %>
				</td>
			</tr>
		</table>
	</div>
	<table class="buttons">
		<tr>
			<td><button type="reset" id="form-discard">Discard changes</button></td>
			<td><button type="submit" id="form-submit"><%-action.firstUpper() %></button></td>
		</tr>
	</table>
	<% if (values.id) { %>
	<input type="hidden" id="uid" name="uid" value="<%-values.id%>" />
	<% } %>
</form>
<a class="invisible" id="overlay-close" href="">Close</a>