<!--Form for signing in users.-->
<form id="changepasswordform" class="userform" action="#">
	<table>
		<tr>
			<td><label for="username">Username</label></td>
			<td><%-user.name ? user.name : ''%><td>
		</tr>
		<tr>
			<td><label for="old_password">Old password</label></td>
			<td><input tabindex="2" type="password" name="old_password" id="old_password"></td>
			<td><a class="focus" href="#" data-target="#old_password"><span>Focus</span></a></td>
			<td><a class="erase" href="#" data-target="#old_password"><span>Erase</span></a></td>
		</tr>
		<tr>
			<td><label for="new_password">New password</label></td>
			<td><input tabindex="2" type="password" name="new_password" id="new_password"></td>
			<td><a class="focus" href="#" data-target="#new_password"><span>Focus</span></a></td>
			<td><a class="erase" href="#" data-target="#new_password"><span>Erase</span></a></td>
		</tr>
		<tr>
			<td><label for="new_password_2">New password again</label></td>
			<td><input tabindex="2" type="password" name="new_password_2" id="new_password_2"></td>
			<td><a class="focus" href="#" data-target="#new_password_2"><span>Focus</span></a></td>
			<td><a class="erase" href="#" data-target="#new_password_2"><span>Erase</span></a></td>
		</tr>
		<tr><td colspan="2"><input type="submit" id="login-submit" value="Change password"></td></tr>
	</table>
</form>
<div id="keyboard-container">
	<ul id="keyboard" class="keyboard">
		<li class="symbol"><span class="off">`</span><span class="on">~</span></li>
		<li class="symbol"><span class="off">1</span><span class="on">!</span></li>
		<li class="symbol"><span class="off">2</span><span class="on">@</span></li>
		<li class="symbol"><span class="off">3</span><span class="on">#</span></li>
		<li class="symbol"><span class="off">4</span><span class="on">$</span></li>
		<li class="symbol"><span class="off">5</span><span class="on">%</span></li>
		<li class="symbol"><span class="off">6</span><span class="on">^</span></li>
		<li class="symbol"><span class="off">7</span><span class="on">&amp;</span></li>
		<li class="symbol"><span class="off">8</span><span class="on">*</span></li>
		<li class="symbol"><span class="off">9</span><span class="on">(</span></li>
		<li class="symbol"><span class="off">0</span><span class="on">)</span></li>
		<li class="symbol"><span class="off">-</span><span class="on">_</span></li>
		<li class="symbol"><span class="off">=</span><span class="on">+</span></li>
		<li class="backspace lastitem action">&#8592;</li>
		<li class="tab action">tab</li>
		<li class="letter">q</li>
		<li class="letter">w</li>
		<li class="letter">e</li>
		<li class="letter">r</li>
		<li class="letter">t</li>
		<li class="letter">y</li>
		<li class="letter">u</li>
		<li class="letter">i</li>
		<li class="letter">o</li>
		<li class="letter">p</li>
		<li class="symbol"><span class="off">[</span><span class="on">{</span></li>
		<li class="symbol"><span class="off">]</span><span class="on">}</span></li>
		<li class="symbol lastitem"><span class="off">\</span><span class="on">|</span></li>
		<li class="capslock action">caps</li>
		<li class="letter">a</li>
		<li class="letter">s</li>
		<li class="letter">d</li>
		<li class="letter">f</li>
		<li class="letter">g</li>
		<li class="letter">h</li>
		<li class="letter">j</li>
		<li class="letter">k</li>
		<li class="letter">l</li>
		<li class="symbol"><span class="off">;</span><span class="on">:</span></li>
		<li class="symbol"><span class="off">'</span><span class="on">&quot;</span></li>
		<li class="return lastitem action">&#8629;</li>
		<li class="left-shift shift action">shift</li>
		<li class="letter">z</li>
		<li class="letter">x</li>
		<li class="letter">c</li>
		<li class="letter">v</li>
		<li class="letter">b</li>
		<li class="letter">n</li>
		<li class="letter">m</li>
		<li class="symbol"><span class="off">,</span><span class="on">&lt;</span></li>
		<li class="symbol"><span class="off">.</span><span class="on">&gt;</span></li>
		<li class="symbol"><span class="off">/</span><span class="on">?</span></li>
		<li class="right-shift shift lastitem action">shift</li>
		<li class="space lastitem">&nbsp;</li>
	</ul>
</div>
