<tr id="<%-id%>">
	<td>
		<select id="<%-cellid%>" name="<%-cellid%>">
			<% groupModel.each(function (group) {%>
				<option value="<%-group.get('id')%>" <%=selected == group.get('id') ? 'selected="selected"' : ''%>><%-group.get('id')%></option>
			<%});%>
		</select>
	</td>
	<td><a href="#" class="ui-icon ui-icon-minusthick deleterow" id="<%-id%>-delete-row" data-id="<%-id%>">Delete</a></td>
</tr>