<form id="framework-configuration-form">
	<div class="inside-wrapper">
		<table class="normalrightcolumn">
			<tr>
				<th><label for="framework_name">Framework name</label></th>
				<td><input type="text" id="framework_name" name="framework_name" value="<%-values.framework_name ? values.framework_name : ''%>" /></td>
			</tr>
			<tr>
				<th><label for="configuration_name">Name</label></th>
				<td><input type="text" id="configuration_name" name="configuration_name" value="<%-values.configuration_name ? values.configuration_name : ''%>" /></td>
			</tr>
			<tr>
				<th><label for="parameters">Parameters</label></th>
				<td><textarea class="codemirror" id="parameters" name="parameters" cols="30" rows="4"><%= values.parameters ? values.parameters : ''%></textarea></td>
			</tr>
		</table>
	</div>
	<table class="buttons">
		<tr>
			<td><button type="reset" id="form-discard">Discard changes</button></td>
			<td><button type="submit" id="form-submit"><%-action.firstUpper() %></button></td>
		</tr>
	</table>
	<% if (values.id) { %>
	<input type="hidden" id="fcid" name="fcid" value="<%-values.id%>" />
	<% } %>
</form>
<a class="invisible" id="overlay-close" href="">Close</a>