$def with (server_address, client_authentication=0, id='', password='')
<!DOCTYPE html>
<html lang="cs">
	<head>
		<meta charset="utf-8">
		<meta name="author" content="Jirka Chadima">
		<title>claun 2.0</title>
		<link href="http://fonts.googleapis.com/css?family=Signika+Negative:300,400,600,700&subset=latin-ext,latin" rel="stylesheet" type="text/css">
		<link type="text/css" href="static/css/cupertino/jquery-ui-1.8.16.custom.css" rel="jQuery UI Stylesheet">
		<link type="text/css" href="static/css/base.css" rel="Stylesheet">

		<script type="text/javascript">
			// Our own namespace, needs to be filled here
			var Claun = {};
			// Base URL for our server requests. It is taken from the config.yaml.
			// Not-so-secret client credentials that were set also on the server. If they are wrong, no user can login using this client. They are taken from the config.yaml.
			Claun.configuration = {
				baseurl: "$server_address",
                client_authentication: Boolean($client_authentication),
				client_id: "$id",
				client_password: "$password",
				console: true
			};
		</script>

		<script type="text/javascript" src="static/js/vendor/jquery/jquery-1.7.1.min.js"></script>
		<script type="text/javascript" src="static/js/vendor/underscore/underscore-min.js"></script>
		<script type="text/javascript" src="static/js/vendor/backbone/backbone-min.js"></script>

		<script type="text/javascript" data-main="static/js/main" src="static/js/vendor/require/require.min.js"></script>
	</head>
	<body>
		<h1>claun 2.0</h1>
		<div id="message"></div> <!--Box for info/error/status messages-->
		<div id="loading">&#64;</div><!--Global loader icon displayed when some background operation is running-->
		<div id="container"></div><!--Main data container-->
		<div id="overlay"></div><!--Modal overlay element that should contain all dialogues/pop-ups-->
		<div id="login"></div><!--Modal overlay for login form-->
	</body>
</html>
