<form id="node-form">
	<div class="inside-wrapper">
		<table class="normalrightcolumn nodeform">
			<tr>
				<th><label for="hostname">Hostname</label></th>
				<td><input type="text" id="hostname" name="hostname" value="<%-values.hostname ? values.hostname : ''%>" /></td>
			</tr>
			<tr class="even">
				<th><label for="ip">IP address</label></th>
				<td><input type="text" id="ip" name="ip" value="<%-values.ip ? values.ip : ''%>" /></td>
			</tr>
			<tr>
				<th><label for="port">Port</label></th>
				<td><input type="text" id="port" name="port" value="<%-values.port ? values.port : ''%>" /></td>
			</tr>
			<tr class="even">
				<th><label for="fqdn">Fully qualified domain name</label></th>
				<td><input type="text" id="fqdn" name="fqdn" value="<%-values.fqdn ? values.fqdn : ''%>" /></td>
			</tr>
			<tr class="closable">
				<th><label for="platforms">Platforms</label></th>
				<td>
					<p class="allwidth"><a href="#" id="add-platform">Add another platform</a></p>
					<table id="platforms">
					</table>
				</td>
			</tr>
			<tr class="closable even">
				<th><label for="projections">Projections</label></th>
				<td>
					<table class="inner projections">
						<tr>
							<th>Projection</th>
							<th>Settings</th>
						</tr>
						<% _.each(projections, function (p) { %>
						<tr>
							<td>
								<label for="projections-is-<%-p%>"><%-p%></label><input <%= values.projections !== undefined && values.projections[p] !== undefined ? 'checked="checked"' : ''%> class="notextchange projections-is" type="checkbox" id="projections-is-<%-p%>" name="projections-is-<%-p%>" value="true" />
							</td>
							<td>
								<div class="projections-configuration-details" id="projections-configuration-details-<%-p%>">
									<p class="allwidth"><label for="projections-<%-p%>"><%-p.firstUpper() %> configuration</label> (<a href="#" id="add-projections-<%-p%>">Add another attribute</a>)</p>
									<table class="inner" id="projections-<%-p%>">
										<tr>
											<th>Key</th>
											<th>Value</th>
										</tr>
									</table>
								</div>
							</td>
						</tr>
						<% }); %>
					</table>
				</td>
			</tr>
			<tr>
				<th>Connected?</th>
				<td><% if(values.connected || values.connected === undefined) {%>
					<label for="connected">YES</label><input type="checkbox" value="true" name="connected" id="connected" checked="checked" />
					<% } else { %>
					<label for="connected">NO</label><input type="checkbox" value="true" name="connected" id="connected" />
					<% } %>
				</td>
			</tr>
		</table>
	</div>
	<table class="buttons">
		<tr>
			<td><button type="reset" id="form-discard">Discard changes</button></td>
			<td><button type="submit" id="form-submit"><%-action.firstUpper() %></button></td>
		</tr>
	</table>
	<% if (values.id) { %>
	<input type="hidden" id="nid" name="nid" value="<%-values.id%>" />
	<% } %>
</form>
<a class="invisible" id="overlay-close" href="">Close</a>