<select id="<%- data.id %>" name="<%- data.id %>">
<% _.each(data.values, function (v) { %>
	<option <%= _.escape(value) == _.escape(v) ? 'selected="selected"' : "" %> value="<%-v%>"><%-v%></option>
<% });%>
</select>