<tr id="<%-id%>">
	<td><input id="<%-cellid%>[key]" name="<%-cellid%>[key]" type="text" value="<%-key ? key : ''%>" /></td>
	<td><input style="float:left" id="<%-cellid%>[value]" name="<%-cellid%>[value]" type="text" value="<%-value ? value : ''%>" /><a href="#" class="ui-icon ui-icon-minusthick deleterow" id="<%-id%>-delete-row" data-id="<%-id%>">Delete</a></td>
</tr>