<form id="parameter-form">
	<div class="inside-wrapper">
		<%=parameter%>
	</div>
	<table class="buttons">
		<tr>
			<td><button type="reset" id="form-discard">Discard changes</button></td>
			<td><button type="submit" id="form-submit"><%-action.firstUpper() %></button></td>
		</tr>
	</table>
	<% if (values.id) { %>
	<input type="hidden" id="pid" name="pid" value="<%-values.id%>" />
	<% } %>
</form>
<a class="invisible" id="overlay-close" href="">Close</a>