/* http://almaer.com/blog/creating-custom-scrollbars-with-css-how-css-isnt-great-for-every-task */
::-webkit-scrollbar {
	width: 30px;
	height: 30px;
}

::-webkit-scrollbar-button:start:decrement,
::-webkit-scrollbar-button:end:increment {
	display: block;
}

::-webkit-scrollbar-button:vertical:start:increment,
::-webkit-scrollbar-button:vertical:end:decrement {
	display: none;
}

::-webkit-scrollbar-button {
	height: 20px;
	width: 30px;
}

::-webkit-scrollbar-button:vertical:start:decrement {
	background: #f2f5f7 url('../imgs/arrow-up.png') bottom left no-repeat;
}

::-webkit-scrollbar-button:vertical:end:increment {
	background: #f2f5f7 url('../imgs/arrow-down.png') top left no-repeat;
}


::-webkit-scrollbar-track-piece {
	background-color: #f2f5f7;

}

::-webkit-scrollbar-thumb {
	background: rgb(174,208,234); /* Old browsers */
	background: -moz-linear-gradient(left,  rgba(174,208,234,1) 0%, rgba(211,239,255,1) 49%, rgba(215,235,249,1) 100%); /* FF3.6+ */
	background: -webkit-gradient(linear, left top, right top, color-stop(0%,rgba(174,208,234,1)), color-stop(49%,rgba(211,239,255,1)), color-stop(100%,rgba(215,235,249,1))); /* Chrome,Safari4+ */
	background: -webkit-linear-gradient(left,  rgba(174,208,234,1) 0%,rgba(211,239,255,1) 49%,rgba(215,235,249,1) 100%); /* Chrome10+,Safari5.1+ */
	background: -o-linear-gradient(left,  rgba(174,208,234,1) 0%,rgba(211,239,255,1) 49%,rgba(215,235,249,1) 100%); /* Opera 11.10+ */
	background: -ms-linear-gradient(left,  rgba(174,208,234,1) 0%,rgba(211,239,255,1) 49%,rgba(215,235,249,1) 100%); /* IE10+ */
	background: linear-gradient(left,  rgba(174,208,234,1) 0%,rgba(211,239,255,1) 49%,rgba(215,235,249,1) 100%); /* W3C */
	filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='#aed0ea', endColorstr='#d7ebf9',GradientType=1 ); /* IE6-9 */

}

::-webkit-scrollbar-resizer {
	background-color: gray;
}
