<table class="slider">
	<tr>
		<th><label for="<%-cellid%>-min">Minimal</label></th>
		<td><input type="text" name="<%-cellid%>[min]" id="<%-cellid%>[min]" value="<%-values.min !== undefined ? values.min : ''%>" /></td>
	</tr>
	<tr>
		<th><label for="<%-cellid%>-max">Maximum</label></th>
		<td><input type="text" name="<%-cellid%>[max]" id="<%-cellid%>[max]" value="<%-values.max !== undefined ? values.max : ''%>"/></td>
	</tr>
	<tr>
		<th><label for="<%-cellid%>-default">Default</label></th>
		<td><input type="text" name="<%-cellid%>[default]" id="<%-cellid%>[default]" value="<%-values['default'] !== undefined ? values['default'] : ''%>"/></td>
	</tr>
</table>