@import url("webkit-scrollbars.css");
@import url("keyboard.css");
@import url("codemirror.css");
@import url("cupertino/jquery.ui.selectmenu.css");

* {
	font-family: 'Signika Negative', sans-serif;
	-moz-user-select: none;
	-khtml-user-select: none;
	-webkit-user-select: none;
	user-select: none;
}

table thead, table tr:nth-child(even) {
	background-color: #e3f1fa;
}

input, textarea {
	-moz-user-select: text;
	-khtml-user-select: text;
	-webkit-user-select: text;
	user-select: text;
}

h1 {
	margin: 15px 0 8px 0;
	padding: 0;
}

.empty {
	background: red;
}

.cycle img {
	padding: 10px;
	border: 1px solid black;
}

.allwidth {
	text-align: center;
}

p.allwidth {
	margin: 0;
	padding: 0;
}


#overlay, #message, #loading {
	display: none;
}

#loading, .loader {
	width: 25px;
	height: 25px;
	background: url('../imgs/loader.gif') center no-repeat;
	font-size: 0.1px;
}

#loading {
	position: absolute;
	bottom: 8px;
	right: 8px;
}

.info {
	border: 2px solid navy !important;
	opacity: .70;
	filter:Alpha(Opacity=70);
}

.error {
	border: 2px solid red !important;
	opacity: .70;
	filter:Alpha(Opacity=70);
}

.warning {
	border: 2px solid chocolate !important;
	opacity: .70;
	filter:Alpha(Opacity=70);
}

.success {
	border: 2px solid greenyellow !important;
	opacity: .70;
	filter:Alpha(Opacity=70);
}

.tooltip {
	border: 2px solid #2694e8;
	background: #e3f1fa;
	position: absolute;
	z-index: 1080;
	padding: 0.4em;
}

.slider-wrapper {
	position: relative;
}

.slider-wrapper .slider-input {
	font-family: 'Asap', sans-serif;
	font-weight: bold;
	text-align: left;
	position: absolute;
	top: -0.3em;
	left: 0;
	border: 0;
	width: 4em;
	background: none;
}

.slider-wrapper .slider {
	position: relative;
	margin-left: 4em;
	margin-right: 1em;
}

.userform {
	width: 60%;
	margin: auto;
	text-align: center;
}

.userform table {width: 100%;}

.userform tr td {
	text-align: right;
	width: 50%;
}

.userform tr td + td {
	text-align: left;
	width: 50%;
}

.userform tr:last-child td {
	text-align: center;
}

.userform input[type=submit] {
	margin: auto;
}

.userform legend {
	padding: 0 0.8em;
}

#keyboard-container {
	margin: 60px auto;
	width: 688px;
}

.userform input[type=text], .userform input[type=password] {
	border: 1px black solid;
	margin-right: 10px;
	font-size: 110%;
}

.userform input.focus[type=text], .userform input.focus[type=password]  {
	border-right: 10px solid black;
	margin-right: 0;
}

a.erase, a.focus {
	display: inline-block;
	width: 32px;
	height: 32px;
}

a.erase span, a.focus span {
	display: none;
}

a.erase {
	background: url('../imgs/erase.png') center center no-repeat;
}

a.focus {
	background: url('../imgs/focus.png') center center no-repeat;
	padding-right: 1em;
}

/* jquery-ui.googlecode.com/svn/trunk/demos/tabs/vertical.html */
/* Vertical Tabs
----------------------------------*/
.ui-tabs-vertical .ui-tabs-nav { padding: 2px; width: 180px; margin-top: 1em; position: absolute; top: 0; left: 3px;}
.ui-tabs-vertical .ui-tabs-nav li { clear: left; width: 100%; border-right-width: 0 !important; overflow: hidden; border-bottom: 1px solid #aed0ea !important;}
.ui-tabs-vertical .ui-tabs-nav li a { display:block; width: 100%; height: 100%; padding: .5em;}
.ui-tabs-vertical .ui-tabs-nav li.ui-tabs-selected { padding-right: 1px; border-right-width: 1px; border-right-width: 1px; border-bottom-color: #2694e8}
.ui-tabs-vertical .ui-tabs-panel { padding: 0 8px;
				   margin-left: 183px;
				   border: 1px solid #2694e8;
				   overflow: auto;
}

.ui-tabs-panel h2 {
	display: none;
}

.invisible {
	display: none;
}

#overlay .loader, .ui-tabs-panel .loader {
	margin: auto;
	margin-top: 10%;
}


#container table {
	width: 100%;
	border-collapse: collapse;
	margin-top: 1em;
}

#container table td, #container table th {
	border-right: 1px solid  #2694e8;
	padding: 0.4em;
	/*width: 30%;*/
}

#container table tr td:last-child, #container table tr th:last-child {
	border-right: 0;
}

#container table th {
	font-weight: bold;
	text-align: right;
	width: 25%;
}

#container table thead {
	border-bottom: 1px solid  #2694e8;
}

#container img {
	margin: auto;
	display: block;
}

#container .overview th {
	text-align: center;
}

button.large {
	margin-top: 2em;
	font-size: larger;
	padding: 0.5em
}

#applications .wrapper {
	margin: 1em -0.5em 1em 0em;
}

#applications table {
	width: 100%;
	table-layout: fixed;
	border-collapse: collapse;
}

#applications tr {
	background: none;
}

#applications table td {
	border: none;
	vertical-align: top;
	padding: 0 0.5em 0.5em 0em;
}

#applications .application > a {
	display: block;
	background-color: white;
	border: 1px solid #2694e8;
	height: 15em;
}

#applications .application h3 {
	margin: 0;
	padding: 0.4em;
}

#applications .cycle {
	margin: auto;
	max-width: 220px;
}

#overlay {
	overflow: hidden;
}

#overlay .inside-wrapper {
	height: 800px;
	overflow-y: scroll;
}

#overlay table {
	width: 100%;
	border-collapse: collapse;
	margin-top: 1em;
}

#overlay table.shortrightcolumn tr td:last-child {
	font-weight: bold;
	width: 5%;
}

#overlay table td, #overlay table th {
	padding: 0.4em;
}

#overlay table th {
	font-weight: bold;
	text-align: right;
	border-right: 1px solid #2694e8;
	width: 40%;
	max-width: 40%;
}

#overlay table.buttons tr td {
	width: auto;
}

#overlay table.buttons tr td:last-child {
	text-align: right;
}

#overlay table.inner {
	margin: 0;
	padding: 0;
}

#parameter-form table.inner table th, #overlay table.inner th {
	font-weight: normal;
	border: none;
	text-align: left;
	margin-bottom: 0;
	padding-bottom: 0;
	width: auto;
	vertical-align: top;
}

#parameter-form table.inner th {
	font-weight: bold;
	text-align: right;
	border-right: 1px solid #2694e8;
	width: 40%;
	max-width: 40%;
}


#overlay table.inner.vertical th {
	text-align: right;
}

#overlay table.inner.horizontal th {
	text-align: left;
}

#overlay table.inner td {
	margin-top: 0;
	padding-top: 0;
	width: auto;
	vertical-align: top;
}

#overlay #application-form .parameter-table {
	border-top: 1px solid #2694E8;
	margin: 8px 0;
}

#overlay #application-form .parameter-table tr:first-child input {
	margin-top: 8px;
}

#overlay #application-form .parameter-table tr:first-child th>label {
	display: block;
	margin-top: 8px;
}

#overlay #node-form .projections>tbody>tr th:last-child, #overlay #application-form .platforms>tbody>tr th:last-child {
	width: 70%;
}

#overlay table.slider tbody tr th, #overlay table.boolean tbody tr th {
    width: 20%;
}

#overlay #node-form .deleterow, #overlay #parameter-form .deleterow, #overlay #application-form .deleterow {
	display: inline-block;
	vertical-align: middle;
	margin-left: 4px;
}

.closable>th:before {
	content: '- ';
}

.closed>th:before {
	content: '+ ';
}

#overlay #node-form table th, #overlay #application-form table th {
	vertical-align: top;
	width: 30%;
}

#overlay #parameter-form .parameter-settings tr td:last-child {
	width: 20%;
}



#application-info {
	margin-top: 2em;
}

#application-info table {
	width: 100%;
	height: 100%;
}

#application-info tr {
	vertical-align: top;
}

#application-info #application-images {
	position: relative;
	height: 100%;
}

#application-info h3 {
	margin-top: 0;
}

#application-info #application-description {
	text-align: justify;
	width: 45%;
	padding-right: 2em;
	border-right: 1px solid #2694e8;
	padding-left: 0;
}

#application-info #application-description p {
	font-size: smaller;
}

#application-info #application-images {
	width: 50%;
}

#application-info #application-images h3 {
	display: none;
}

#image-editor table th {
	border: none;
	text-align: left;
}

#image-editor .left {
	float: left;
	width: 49%;
	height: 100%;
	padding-right: 1em;
	margin-right: 1em;
	border-right: 1px solid #2694e8;
}

#image-editor .right {
	width: 48%;
	float: right;
}


#applist-form-launch {
	font-size: 130%;
}

#running-box {
	position: absolute;
	top: 8px;
	right: 8px;
	border: 1px solid black;
	padding: 0 1em;
}

#running-box p {
	margin: 0.3em 0;
}

#running-box p > span {
	margin-right: 2em;
}

#running-box button {
	margin-left: 6em;
	font-size: smaller;
}

.buttons  {
	list-style-type: none;
	margin: 0;
	padding: 0;
	margin-top: 0.5em;
	position: absolute;
	bottom: 8px;
	left: 8px;
}

.buttons li {
	float: left;
	font-size: 80%;
}

.buttons button {
	margin: 0 4px 0 0;
}
